@@ -7,6 +7,7 @@ import sys
 import itertools
 import inspect
 import warnings
+import importlib
 from importlib import import_module
 
 from configparser import ConfigParser
@@ -26,6 +27,8 @@ def _get_parser():
 
     parser = ArgumentParser()
     parser.usage = "testflo [options]"
+    parser.add_argument('--version', action='store_true', dest='version',
+                        help="Display the version number and exit.")
     parser.add_argument('-c', '--config', action='store', dest='cfg',
                         metavar='FILE',
                         help='Path of config file where preferences are specified.')
@@ -38,10 +41,10 @@ def _get_parser():
                              'the quicktests.in file.')
 
     parser.add_argument('-n', '--numprocs', type=int, action='store',
-                        dest='num_procs', metavar='NUM_PROCS',
-                        help='Number of processes to run. By default, this will '
-                             'use the number of CPUs available.  To force serial'
-                             ' execution, specify a value of 1.')
+                        dest='num_procs', metavar='NUM_TEST_PROCS',
+                        help='Number of concurrent test processes to run. By default, this will '
+                             'use the number of virtual processors available.  To force tests to '
+                             'run consecutively, specify a value of 1.')
     parser.add_argument('-o', '--outfile', action='store', dest='outfile',
                         metavar='FILE', default='testflo_report.out',
                         help='Name of test report file.  Default is testflo_report.out.')
@@ -98,8 +101,12 @@ def _get_parser():
     parser.add_argument('--show_skipped', action='store_true', dest='show_skipped',
                         help="Display a list of any skipped tests in the summary.")
 
+    parser.add_argument('--disallow_skipped', action='store_true', dest='disallow_skipped',
+                        help="Return exit code 2 if no tests failed but some tests are skipped.")
+
     parser.add_argument('tests', metavar='test', nargs='*',
-                        help='A test method, test case, module, or directory to run.')
+                        help='A test method, test case, module, or directory to run. If not '
+                             'supplied, the current working directory is assumed.')
 
     parser.add_argument('-m', '--match', '--testmatch', action='append', dest='test_glob',
                         metavar='GLOB',
@@ -250,10 +257,13 @@ def fpath2modpath(fpath):
         pnames = []
     else:
         pnames = [splitext(basename(fpath))[0]]
+
     path = dirname(abspath(fpath))
+
     while isfile(join(path, '__init__.py')):
-            path, pname = split(path)
-            pnames.append(pname)
+        path, pname = split(path)
+        pnames.append(pname)
+
     return '.'.join(pnames[::-1])
 
 
@@ -273,39 +283,18 @@ def find_module(name):
     given module name, or None if it can't be found. The
     file must be an uncompiled Python (.py) file.
     """
-
-    nameparts = name.split('.')
-
-    endings = [join(*nameparts)]
-    endings.append(join(endings[0], '__init__.py'))
-    endings[0] += '.py'
-
-    for entry in sys.path:
-        for ending in endings:
-            f = join(entry, ending)
-            if isfile(f):
-                return f
-    return None
+    try:
+        info = importlib.util.find_spec(name)
+    except ImportError:
+        info = None
+    if info is not None:
+        return info.origin
 
 
-def get_module(fname):
-    """Given a filename or module path name, return a tuple
-    of the form (filename, module).
-    """
+_mod2file = {}  # keep track of non-pkg files to detect and flag dups
 
-    if fname.endswith('.py'):
-        modpath = fpath2modpath(fname)
-        if not modpath:
-            raise RuntimeError("can't find module %s" % fname)
-    else:
-        modpath = fname
-        fname = find_module(modpath)
-
-        if not fname:
-            raise ImportError("can't import %s" % modpath)
-
-    start_coverage()
 
+def try_import(fname, modpath):
     try:
         mod = import_module(modpath)
     except ImportError:
@@ -325,6 +314,48 @@ def get_module(fname):
             del sys.modules[modpath]
         finally:
             sys.path = oldpath
+
+    return mod
+
+
+def get_module(fname):
+    """Given a filename or module path name, return a tuple
+    of the form (filename, module).
+    """
+
+    if fname.endswith('.py'):
+        modpath = fpath2modpath(fname)
+        if not modpath:
+            raise RuntimeError("can't find module %s" % fname)
+
+        if modpath in _mod2file:
+            old = _mod2file[modpath]
+            if old != fname:
+                raise RuntimeError("module '%s' was already imported earlier from file '%s' so "
+                                    "it can't be imported from file '%s'. To fix this problem, "
+                                    "either rename the file or add the file to a python package "
+                                    "so the resulting module path will be unique." %
+                                    (modpath, old, fname))
+        else:
+            _mod2file[modpath] = fname
+
+    else:
+        modpath = fname
+        fname = find_module(modpath)
+
+        if fname:
+            _mod2file[modpath] = fname
+        else:
+            # check for a non-pkg module
+            if modpath in _mod2file:
+                fname = _mod2file[modpath]
+            else:
+                raise ImportError("can't import %s" % modpath)
+
+    start_coverage()
+
+    try:
+        mod = try_import(fname, modpath)
     finally:
         stop_coverage()
 
